@@ -1,30 +1,7 @@
-/* Spa libcamera Source
- *
- * Copyright (C) 2020, Collabora Ltd.
- *     Author: Raghavendra Rao Sidlagatta <raghavendra.rao@collabora.com>
- * Copyright (C) 2021 Wim Taymans <wim.taymans@gmail.com>
- *
- * libcamera-source.cpp
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* Spa libcamera source */
+/* SPDX-FileCopyrightText: Copyright © 2020 Collabora Ltd. */
+/*                         @author Raghavendra Rao Sidlagatta <raghavendra.rao@collabora.com> */
+/* SPDX-License-Identifier: MIT */
 
 #include <stddef.h>
 #include <sys/types.h>
@@ -49,6 +26,7 @@
 #include <spa/node/keys.h>
 #include <spa/param/video/format-utils.h>
 #include <spa/param/param.h>
+#include <spa/param/latency-utils.h>
 #include <spa/control/control.h>
 #include <spa/pod/filter.h>
 
@@ -82,14 +60,6 @@ struct buffer {
 	void *ptr;
 };
 
-#define MAX_CONTROLS	64
-
-struct control {
-	uint32_t id;
-	uint32_t ctrl_id;
-	double value;
-};
-
 struct port {
 	struct impl *impl;
 
@@ -100,16 +70,14 @@ struct port {
 
 	uint32_t memtype = 0;
 
-	struct control controls[MAX_CONTROLS];
-	uint32_t n_controls = 0;
-
 	struct buffer buffers[MAX_BUFFERS];
 	uint32_t n_buffers = 0;
 	struct spa_list queue;
 	struct spa_ringbuffer ring = SPA_RINGBUFFER_INIT();
 	uint32_t ring_ids[MAX_BUFFERS];
 
-	static constexpr uint64_t info_all = SPA_PORT_CHANGE_MASK_FLAGS | SPA_PORT_CHANGE_MASK_PARAMS;
+	static constexpr uint64_t info_all = SPA_PORT_CHANGE_MASK_FLAGS |
+		SPA_PORT_CHANGE_MASK_PROPS | SPA_PORT_CHANGE_MASK_PARAMS;
 	struct spa_port_info info = SPA_PORT_INFO_INIT();
 	struct spa_io_buffers *io = nullptr;
 	struct spa_io_sequence *control = nullptr;
@@ -119,7 +87,8 @@ struct port {
 #define PORT_IO		3
 #define PORT_Format	4
 #define PORT_Buffers	5
-#define N_PORT_PARAMS	6
+#define PORT_Latency	6
+#define N_PORT_PARAMS	7
 	struct spa_param_info params[N_PORT_PARAMS];
 
 	uint32_t fmt_index = 0;
@@ -137,6 +106,7 @@ struct port {
 		params[PORT_IO] = SPA_PARAM_INFO(SPA_PARAM_IO, SPA_PARAM_INFO_READ);
 		params[PORT_Format] = SPA_PARAM_INFO(SPA_PARAM_Format, SPA_PARAM_INFO_WRITE);
 		params[PORT_Buffers] = SPA_PARAM_INFO(SPA_PARAM_Buffers, 0);
+		params[PORT_Latency] = SPA_PARAM_INFO(SPA_PARAM_Latency, SPA_PARAM_INFO_READ);
 
 		info.flags = SPA_PORT_FLAG_LIVE | SPA_PORT_FLAG_PHYSICAL | SPA_PORT_FLAG_TERMINAL;
 		info.params = params;
@@ -175,6 +145,8 @@ struct impl {
 	struct spa_io_position *position = nullptr;
 	struct spa_io_clock *clock = nullptr;
 
+	struct spa_latency_info latency[2];
+
 	std::shared_ptr<CameraManager> manager;
 	std::shared_ptr<Camera> camera;
 
@@ -431,15 +403,14 @@ static int impl_node_send_command(void *object, const struct spa_command *comman
 	return 0;
 }
 
-static const struct spa_dict_item info_items[] = {
-	{ SPA_KEY_DEVICE_API, "libcamera" },
-	{ SPA_KEY_MEDIA_CLASS, "Video/Source" },
-	{ SPA_KEY_MEDIA_ROLE, "Camera" },
-	{ SPA_KEY_NODE_DRIVER, "true" },
-};
-
 static void emit_node_info(struct impl *impl, bool full)
 {
+	static const struct spa_dict_item info_items[] = {
+		{ SPA_KEY_DEVICE_API, "libcamera" },
+		{ SPA_KEY_MEDIA_CLASS, "Video/Source" },
+		{ SPA_KEY_MEDIA_ROLE, "Camera" },
+		{ SPA_KEY_NODE_DRIVER, "true" },
+	};
 	uint64_t old = full ? impl->info.change_mask : 0;
 	if (full)
 		impl->info.change_mask = impl->info_all;
@@ -453,10 +424,15 @@ static void emit_node_info(struct impl *impl, bool full)
 
 static void emit_port_info(struct impl *impl, struct port *port, bool full)
 {
+	static const struct spa_dict_item info_items[] = {
+		{ SPA_KEY_PORT_GROUP, "stream.0" },
+	};
 	uint64_t old = full ? port->info.change_mask : 0;
 	if (full)
 		port->info.change_mask = port->info_all;
 	if (port->info.change_mask) {
+		struct spa_dict dict = SPA_DICT_INIT_ARRAY(info_items);
+		port->info.props = &dict;
 		spa_node_emit_port_info(&impl->hooks,
 				SPA_DIRECTION_OUTPUT, 0, &port->info);
 		port->info.change_mask = old;
@@ -624,6 +600,15 @@ next:
 			return 0;
 		}
 		break;
+	case SPA_PARAM_Latency:
+		switch (result.index) {
+		case 0: case 1:
+			param = spa_latency_build(&b, id, &impl->latency[result.index]);
+			break;
+		default:
+			return 0;
+		}
+		break;
 	default:
 		return -ENOENT;
 	}
@@ -914,7 +899,7 @@ static int impl_node_process(void *object)
 }
 
 static const struct spa_node_methods impl_node = {
-	SPA_VERSION_NODE_METHODS,
+	.version = SPA_VERSION_NODE_METHODS,
 	.add_listener = impl_node_add_listener,
 	.set_callbacks = impl_node_set_callbacks,
 	.sync = impl_node_sync,
@@ -984,6 +969,9 @@ impl::impl(spa_log *log, spa_loop *data_loop, spa_system *system,
 	info.flags = SPA_NODE_FLAG_RT;
 	info.params = params;
 	info.n_params = N_NODE_PARAMS;
+
+	latency[SPA_DIRECTION_INPUT] = SPA_LATENCY_INFO(SPA_DIRECTION_INPUT);
+	latency[SPA_DIRECTION_OUTPUT] = SPA_LATENCY_INFO(SPA_DIRECTION_OUTPUT);
 }
 
 static size_t
