@@ -498,14 +498,18 @@ static int impl_send_command(void *object, const struct spa_command *command)
 {
 	struct filter *impl = object;
 	struct pw_filter *filter = &impl->this;
+	uint32_t id = SPA_NODE_COMMAND_ID(command);
 
-	switch (SPA_NODE_COMMAND_ID(command)) {
+	pw_log_debug("%p: command %s", impl,
+			spa_debug_type_find_name(spa_type_node_command_id, id));
+
+	switch (id) {
 	case SPA_NODE_COMMAND_Suspend:
 	case SPA_NODE_COMMAND_Flush:
 	case SPA_NODE_COMMAND_Pause:
 		pw_loop_invoke(impl->main_loop,
 			NULL, 0, NULL, 0, false, impl);
-		if (filter->state == PW_FILTER_STATE_STREAMING) {
+		if (filter->state == PW_FILTER_STATE_STREAMING && id != SPA_NODE_COMMAND_Flush) {
 			pw_log_debug("%p: pause", filter);
 			filter_set_state(filter, PW_FILTER_STATE_PAUSED, 0, NULL);
 		}
